	.length 78
	.width  132
;************************************************************************
;									*
;				S208AT1.ASM				*
;									*
;************************************************************************
; C40 test program which only requires C40 internal RAM BLK 0 and 1.
; It should be loaded using the Emulator over the JTAG interface.
; I/O is restricted to the Emulator display/edit facilities.
;
; History:
; 07-Jun-96 SEC Loop around test all c011 links. Requires hardware links:
;		Link0Out 12 -> Link3In 16
;		Link3Out 15 -> Link0In 13
;		Link1Out 04 -> Link2In 02
;		Link2Out 01 -> Link1In 05
;

;************************************************************************
;	EQUATES								*
;************************************************************************

ramblk0_st	.equ	002FF800h	; Start of internal sram
start		.equ	002FF900h	; Code start address
stack		.equ	002FFE00h	; Stack start address, grows up
ramblk1_end	.equ	002FFFFFh	; End of internal sram

link0		.equ	40000000h	; Link 0 c011
link1		.equ	40000004h	; Link 1 c011
link2		.equ	40000008h	; Link 2 c011
link3		.equ	4000000Ch	; Link 3 c011

led		.equ	40000010h	; Led0

link_din	.equ	0		; c011 data in
link_dout	.equ	1		; c011 data out
link_statin	.equ	2		; c011 data in status
link_statout	.equ	3		; c011 data out status

link_test	.equ	link0		; link to loopback


;************************************************************************
;	CODE								*
;************************************************************************

	.global	_main

; The program leaves space for the ID ROM data at ramblk0_st:

;	.asect "_main",start		; Code section, also contains stack
	.text

; Disable interrupts, repeat off, cache off:
_main:	ldi	0,st			; clear status reg
	ldp	@stack,dp		; get stack page
	ldhi	stack/10000h,sp		; set sp msb
	or	stack&0ffffh,sp		; set sp lsb


	ldi	05Ah,r1			; seed data
	ldi	0,r4			; pass count

; Test l0 to l3:
tst03:	ldi	0,ar0			; link 0
	call	lout			; write r1
	ldi	3,ar0			; link 3
	call	lin			; read to r2
	cmpi	r1,r2			; same ?
	bne	error			; no

; Test l3 to l0:
tst30:	ldi	3,ar0			; link 3
	call	lout			; write r1
	ldi	0,ar0			; link 0
	call	lin			; read to r2
	cmpi	r1,r2			; same ?
	bne	error			; no

; Test l1 to l2:
tst12:	ldi	1,ar0			; link 1
	call	lout			; write r1
	ldi	2,ar0			; link 2
	call	lin			; read to r2
	cmpi	r1,r2			; same ?
	bne	error			; no

; Test l2 to l1:
tst21:	ldi	2,ar0			; link 2
	call	lout			; write r1
	ldi	1,ar0			; link 1
 	call	lin			; read to r2
	cmpi	r1,r2			; same ?
	bne	error			; no

; Loop around, with new data:
	addi	1,r1			; inc data
	and	0FFh,r1			; mask to byte
	addi	1,r4			; inc pass count
	call	flash			; flash led
	br	tst03			; loop


; on error halt:
error:	br	error


; Input data to r2:
lin:	call	ladd			; addr link
lin1:	ldi	*ar2,r0			; read status
	tstb	1,r0			; test data in flag
	bz	lin1			; poll if 0
	ldi	*ar0,r2			; read data byte
	and	0FFh,r2			; mask to byte
	rets

; Output data from r1:
lout:	call	ladd			; addr link
lout1:	ldi	*ar3,r0			; read status
	tstb	1,r0			; test data out flag
	bz	lout1			; poll if 0
	sti	r1,*ar1			; write data out
	rets

; Address link registers, link number in ar0:
ladd:	ldp	@link_test,dp		; get link page
	lsh     2,ar0			; mult link num by 4
	ldhi	link_test/10000h,ar1	;
	or	link_test&0FFFFh,ar1	;
	addi	ar1,ar0			; address test link
	ldi	ar0,ar1			; copy
	ldi	ar0,ar2			; copy
	ldi	ar0,ar3			; copy
	addi	link_din,ar0		; addr data in
	addi	link_dout,ar1		; addr data out
	addi	link_statin,ar2		; addr data in stat
	addi	link_statout,ar3	; addr data out stat
	rets

; Flash LED using pass count in r4:
flash:  ldhi	led/10000h,ar7		; address led
        or	led&0FFFFh,ar7		;
	ldi	0,r5			; guess led is off
	tstb	8000h,r4		; led on or off ?
	beq	flash1			; off
	ldi	1,r5			; led on
flash1: sti	r5,*ar7			; set LED bit
	rets

	.end

