/*--------------------------------------------------*/
/*                                                  */
/* MEMTEST.C                                        */
/*                                                  */
/* This program tests the ram on the Serial/Timer   */
/* tram card.                                       */
/*                                                  */
/*--------------------------------------------------*/

#include <stdio.h>
#include <iocntrl.h>
#include <process.h>
#include <channel.h>
#include <stdlib.h>
#include <misc.h>
#include <string.h>
#include <process.h>

#include "tramio.h"



#define BUFSIZE		8	/* Data Buffer SIze */
#define RAMSTART	0x8000	/* Start of Ram     */			
#define RAMEND		0xFFFF	/* End of Ram       */



int main(void)
{
	

	Channel	*LinkIn, *LinkOut;
	
	char	user_ip;

	byte	ReadData[BUFSIZE], WriteData[BUFSIZE];

	int	i, j, Error, LinkState, WriteAddress = RAMSTART;

	LinkIn  = get_param(3);
	LinkOut = get_param(4);

	ChanReset(LinkIn);
	ChanInit(LinkIn);
	
	ChanReset(LinkOut);
	ChanInit(LinkOut);
	
	printf("\n\n Serial/Timer Tram, Ram Test\n\n");	
	printf("Press any key to Exit\n\n");
	

	while(((user_ip = pollkey()) == 0xff) && (WriteAddress < RAMEND))
	{

		for(j=0; j<3; j++)
		{
			switch(j)
			{
				case 0:
					for(i=0; i<BUFSIZE; i++)
						WriteData[i] = i;
					break;	
				case 1:
					for(i=0; i<BUFSIZE; i++)
						WriteData[i] = 0x55;
					break;	
				case 2:
					for(i=0; i<BUFSIZE; i++)
						WriteData[i] = 0xAA;
					break;	

			}
			
			if((LinkState = WriteTramMem( LinkOut, WriteAddress, BUFSIZE, WriteData)) == 1)
			{
				printf("Error - Timeout writing data, Reseting Tram\n");
				ResetTram(LinkIn, LinkOut);
			}


			if((LinkState = ReadTramMem(LinkIn, LinkOut, WriteAddress, BUFSIZE, ReadData)) == 1)
			{
				printf("Error - Timeout reading data, Reseting Tram\n");
				ResetTram(LinkIn, LinkOut);

			}
			else if (LinkState == 0)
			{

				/*--------------------------------------*/
				/* Write and Read completed, now verify */
				/*--------------------------------------*/
				printf("%04x  ",WriteAddress);

				for(i=0; i<BUFSIZE; i++)
					printf("%02x",WriteData[i]);

				printf("  ");	

				for(i=0; i<BUFSIZE; i++)
					printf("%02x",ReadData[i]);

				Error = 0;	
				for(i=0; i<BUFSIZE; i++)
					if(ReadData[i] != WriteData[i])
						Error = 1;

				printf("\n");
			
				if(Error)
				{
					printf("Ram Check Error Detected - Press any key to continue\n");
					user_ip = getkey();
				}
			}
			else
				printf("Error - Unexpected Link State = %02x\n",LinkState);
			
		}
			
		/*------------------------------*/	
		/* Increment the Memory Address */
		/*------------------------------*/
		WriteAddress += BUFSIZE;		

	}	

	if(WriteAddress == RAMEND)
		printf("TEST COMPLETED!\n");
		
	printf("Exit from program\n");
	
	exit_terminate(EXIT_SUCCESS);
}
