/*
 * fibtest Test Software
 * Version 2.0
 * This has been fixed from the original by Glasgow University so that
 * the number of passes section works properly.
 */

#define BILL_GATES_FOR_WORLD_DOMINATION 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <ctype.h>                                       
#include <conio.h>

#define DELAY_TIME 50
#define MAX_TRIES 0xFFFF

#define PORT_BASE_A 0x150
#define PORT_BASE_B 0x200

#define DATA_IN_OFFSET 0x00
#define DATA_OUT_OFFSET 0x01
#define DATA_IN_STATUS_OFFSET 0x02
#define DATA_OUT_STATUS_OFFSET 0x03
#define RESET_OFFSET 0x10
#define ANALYSE_OFFSET 0x11
#define ERROR_OFFSET 0x10

#define RESET_BIT 1
#define ANALYSE_BIT 1
#define ERROR_BIT 1

#define DATA_READY_BIT 1

#define FALSE 0
#define TRUE 1

/*
 * We need this if using the Macrosurft compiler
 */
#ifdef BILL_GATES_FOR_WORLD_DOMINATION 
#define outp(port, val) _outp(port, val)
#define inp(port)  _inp(port)
#include <time.h>
void delay(int val)
{
	time_t t0, t1;
	int loop;         
	time(&t0);
	loop=0;
	do{
		time(&t1);
	}while((t1 - t0) != 1);
}
#endif

long num_passes = 1L;
int test_ss = FALSE;

int port_base[2] = {PORT_BASE_A,PORT_BASE_B};
char *port_name[2] = {"A","B"};

int ss_res[2][2] =
{
	{0,1},
	{1,0}
};

void reset_board(int port)
{
	int reset_port,analyse_port,data_out_status,data_in_status;

	reset_port = port_base[port] + RESET_OFFSET;
	analyse_port = port_base[port] + ANALYSE_OFFSET;
	data_in_status = port_base[port] + DATA_IN_STATUS_OFFSET;
	data_out_status = port_base[port] + DATA_OUT_STATUS_OFFSET;
    outp(reset_port,RESET_BIT);
    delay(DELAY_TIME);
    outp(analyse_port,!ANALYSE_BIT);
    delay(DELAY_TIME);
    outp(reset_port,!RESET_BIT);
    outp(data_out_status,0);
    outp(data_in_status,0);
}

void byte_to_link_t(int port,int value,int *ret_code)
{
	int data_out,data_out_status;
    long tries;

	data_out_status = port_base[port] + DATA_OUT_STATUS_OFFSET;
	data_out = port_base[port] + DATA_OUT_OFFSET;
	*ret_code = FALSE;
    for (tries = 0; (tries < MAX_TRIES) && (!(*ret_code)); tries++)
    {
        if ((inp(data_out_status) & DATA_READY_BIT) != 0)
        {
            *ret_code = TRUE;
			outp(data_out,value);
        }
    }
}

void byte_from_link_t(int port,int *value,int *ret_code)
{
	int data_in,data_in_status;
    long tries;

	data_in_status = port_base[port] + DATA_IN_STATUS_OFFSET;
	data_in = port_base[port] + DATA_IN_OFFSET;
	*ret_code = FALSE;
    for (tries = 0; (tries < MAX_TRIES) && (!(*ret_code)); tries++)
    {
		if ((inp(data_in_status) & DATA_READY_BIT) != 0)
        {
            *ret_code = TRUE;
            *value = inp(data_in);
        }
    }
}

void ack_from_link_t(int port,int *ret_code)
{
	int data_out_status;
    long tries;

	data_out_status = port_base[port] + DATA_OUT_STATUS_OFFSET;
	*ret_code = FALSE;
	for (tries = 0; (tries < MAX_TRIES) && (!(*ret_code)); tries++)
    {
        if ((inp(data_out_status) & DATA_READY_BIT) != 0)
        {
            *ret_code = TRUE;
        }
    }
}

int read_number(char *str)
{
    int result = 0;
    int base;
    int carry_on;
    int c;

    if (*str == '#')
    {
        base = 16; str++;
    }
	else
	{
        base = 10;
	}
    carry_on = TRUE;
    while (carry_on)
    {
       c = toupper(*str++);
       if ((c >= '0') && (c <= '9'))
	   {
		   result = (result * base) + c - '0';
	   }
       else
       {
           if (base == 16)
           {
               if ((c >= 'A') && (c <= 'F'))
			   {
                   result = (result * base) + 10 + c - 'A';
			   }
               else
			   {
                   carry_on = FALSE;
			   }
           }
		   else
		   {
               carry_on = FALSE;
		   }
       }
    }
    return (result);
}

void usage(void)
{
    printf("USAGE : fibtest -[abps?h]\n");
    printf("Valid options are :-\n");
    printf("a <num> sets address for board a\n");
    printf("b <num> sets address for board b\n");
	printf("p <num> sets number of passes to perform\n");
    printf("s test subsystem operation\n");
    printf("? or h this message\n");
}

void parse_command_line(int ac,char *av[],int *abort)
{
    char *str;
    int loop;

	for (loop = 0; loop < ac; loop++)
    {
        str = strupr(av[loop]);
        switch (str[0])
        {
			case '/' :
			case '-' :
			{
				switch (toupper(str[1]))
				{
					case 'A' :
					{
						loop++;
						if (loop <= ac)
						{
							port_base[0] = read_number(av[loop]);
						}
						else
						{
							printf("Missing port A number\n");
						}
						break;
					}
					case 'B' :
					{
						loop++;
						if (loop <= ac)
						{
							port_base[1] = read_number(av[loop]);
						}
						else
						{
							printf("Missing port B number\n");
						}
						break;
					}
					case 'P' :
					{
						loop++;
						if (loop <= ac)
						{
                     sscanf(av[loop],"%ld",&num_passes);
						}
						else
						{
							printf("Missing number of passes\n");
						}
						break;
					}
					case 'S' :
					{
						test_ss = TRUE;
						break;
					}
					case '?' :
					case 'H' :
					{
						usage();
						*abort = TRUE;
						break;
					}
					default :
					{
						printf("Unknown Command Line Option %s\n",str);
						break;
					}
				}
			}
            default :
			{
				break;
			}
        }
    }
	if (port_base[0] == port_base[1])
	{
		*abort = TRUE;
		printf("Boards must be at different addresses\n");
	}
}

void test_channel(int port_a,int port_b,int *abort)
{
	int i,success,ri;

	success = TRUE;
	for (i = 0; (i < 256) && success; i++)
	{
		byte_to_link_t(port_a,i,&success);
		if (success)
		{
			byte_from_link_t(port_b,&ri,&success);
			if (!success)
			{
				printf("\nFAILED to receive byte at port %s",
					   port_name[port_b]);
			}
			else if (ri != i)
			{
				printf("\nByte received at port %s is incorrect (%i => %i)",
					   port_name[port_b],i,ri);
			}
			else
			{
				ack_from_link_t(port_a,&success);
				if (!success)
				{
					printf("\nAcknowledge not received by port %s",
						   port_name[port_a]);
				}
			}
		}
		else
		{
			printf("\nFAILED to send byte by port %s",
				   port_name[port_a]);
		}
	}
	if (!success)
	{
		*abort = TRUE;
	}
}

void test_subsystem(int port,int *abort)
{
	int reset_port,analyse_port,error_port;
	int i,j,res[2][2],ok;

	reset_port = port_base[port] + RESET_OFFSET;
	analyse_port = port_base[port] + ANALYSE_OFFSET;
	error_port = port_base[port] + ERROR_OFFSET;
	ok = TRUE;
	for (i = 0; i < 2; i++)
	{
		for (j = 0; j < 2; j++)
		{
			outp(reset_port,i * RESET_BIT);
			outp(analyse_port,j * ANALYSE_BIT);
			delay(DELAY_TIME);
			res[i][j] = inp(error_port) & ERROR_BIT;
			ok = ok && (res[i][j] == ss_res[i][j]);
		}
	}
	if (!ok)
	{
        *abort = TRUE;
		printf("\nSubsystem failure while testing port %s",port_name[port]);
		printf("\n \\A| 0 1");
		printf("\n R\\|");
		printf("\n--------");
		printf("\n 0 | %i %i",res[0][0],res[0][1]);
		printf("\n 1 | %i %i",res[1][0],res[1][1]);
	}
}

int main(int argc,char *argv[])
{
	long p;
	int abort;

	abort = FALSE;
	printf("\nSundance SMT227 Test Routine\n");
   printf("Version 2.0 27/5/1998 M.A.\n");
   printf("(Fixed passes routine)\n\n");
   parse_command_line(argc,argv,&abort);
   if (!abort)
	{
		reset_board(0);
		reset_board(1);
	}
	for (p = 0L; (p < num_passes) && (!abort); p++)
	{
		printf("PASS %li ",p + 1L);
		test_channel(0,1,&abort);
		test_channel(1,0,&abort);
		if (!abort)
		{
			printf("OK\r");
		}
	}
	if (test_ss)
	{
		printf("\n");
		test_subsystem(0,&abort);
		test_subsystem(1,&abort);
		if(!abort)
			printf("Subsystem Passed\n");
	}
	printf("\n");
   return(0);
}
